from mcpi.vec3 import Vec3

import pandas as pd

from mcthings.thing import Thing
from mcthings.scene import Scene
from mcthings.world import World

ABADIA_LEVEL_HEIGHT = 16


class CsvPoints(Thing):
    """
    The format of the CSV file must be:
    Level,X,Z,Y
    """
    file_path = None
    """ Path to the CSV file with the points to build"""
    level = "0"
    """ Which level to read """

    def draw_blocks(self, y, z, axis_x_list, level_height):
        """

        :param y: blocks y-axis position
        :param z: blocks z-axis position
        :param axis_x_list: list of x-axis in which a block must exist
        :param level_height: height in which blocks are built
        :return:
        """

        # Time to draw the blocks (y = height, z = Y are always the same in a group)
        init_y = self.position.y + y
        init_z = self.position.z + z
        # Create walls to the level_height
        end_y = self.position.y + level_height
        end_z = self.position.z + z

        # To detect splits (not incremental by 1 values) in the series order the x-axis positions
        axis_x_list.sort()
        init_split_x = axis_x_list[0]

        for i in range(1, len(axis_x_list)):
            if (axis_x_list[i] - axis_x_list[i-1]) > 1:
                init_x = self.position.x + init_split_x
                end_x = self.position.x + axis_x_list[i-1]
                World.server.postToChat("Building blocks defined in a CSV file (%s, %s, %s)"
                                        % (init_x, init_y, init_z))
                World.server.setBlocks(init_x, init_y, init_z, end_x, end_y, end_z, self.block)

                init_split_x = axis_x_list[i]

        init_x = self.position.x + init_split_x
        end_x = self.position.x + axis_x_list[-1]
        World.server.setBlocks(init_x, init_y, init_z, end_x, end_y, end_z, self.block)

    def build(self):
        # Read the CSV file
        if not self.file_path:
            RuntimeError("Missing file_path param")

        # In order to explore the data pandas helps with group by and others operations
        df = pd.read_csv(self.file_path, delimiter=",")
        df.columns = ['Level', 'X', 'Z', 'Y']

        # Limits of the Abadia: (max_x-min_x, max_y-min_y, max_z-min_z) is the size
        min_y = min(df['Y'])
        max_y = max(df['Y'])
        min_x = min(df['X'])
        max_x = max(df['X'])
        min_z = min(df['Z'])
        max_z = max(df['Z'])
        self._end_position = Vec3(max_x-min_x, max_y-min_y, max_z-min_z)

        # Load each level in a new dataframe
        df0 = df[df['Level'].eq(0)]
        df1 = df[df['Level'].eq(1)]
        df2 = df[df['Level'].eq(2)]

        # Drop Level column not needed anymore
        df0.drop('Level', axis=1, inplace=True)
        df1.drop('Level', axis=1, inplace=True)
        df2.drop('Level', axis=1, inplace=True)

        # Let's analyze the data to find the best way to draw it
        # It seems that all the data has a range of X for a fixed Z and Y
        # Let's confirm it using GROUP BY to analyze this Y,Z groups
        #
        # Blocks needed to build the Levels using horizontal columns along X
        # This is True is horizontal column is continuous but it can have splits
        # total_x_blocks_0 = len(df0.groupby(['Y', 'Z'])['X'].count())  # 828
        # total_x_blocks_1 = len(df1.groupby(['Y', 'Z'])['X'].count())  # 385
        # total_x_blocks_2 = len(df2.groupby(['Y', 'Z'])['X'].count())  # 683
        # In the next df we have the list of Xs per Y and height. With it we can build one or more blocks
        # along the X-axis

        level_height = 0
        for level in [df0, df1, df2]:
            for name, group in level.groupby(['Y', 'Z'])['X']:
                axis_y = name[0]
                axis_z = name[1]
                axis_x_list_offset = group.values.tolist()
                axis_x_list = list(map(lambda x: x - min_x, axis_x_list_offset))

                # fixed position in Minecraft or y and z axis
                y = axis_y - min_y + level_height
                z = axis_z - min_z

                # The blocks along x could be just one or several splitted
                self.draw_blocks(y, z, axis_x_list, level_height)

            level_height += ABADIA_LEVEL_HEIGHT



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author/s (©): Familia de Lorenzo

from mcpi.vec3 import Vec3
from mcthings.scene import Scene
from mcthings.thing import Thing
from mcthings.world import World


class StairsSnail(Thing):
    """ Create snail stairs with sections including steps of width """

    sections = 3
    steps = 5
    width = 5

    def build(self):
        mc = World.server

        direction = 0

        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        for section in range(0, self.sections, 1):
            if direction == 0:
                for eje_y in range(0, self.steps, 1):
                    for eje_x in range(0, self.width, 1):
                        mc.setBlock((init_x + eje_x),
                                    ((self.steps * section) + init_y + eje_y),
                                    (init_z + eje_y) + (direction * self.steps),
                                    self.block)
                direction = 1
            elif direction == 1:
                for eje_y in range(0, self.steps, 1):
                    for eje_x in range(0, self.width, 1):
                        mc.setBlock((init_x + eje_x + self.width),
                                    ((self.steps * section) + init_y + eje_y),
                                    (init_z - eje_y - 1) + (direction * self.steps),
                                    self.block)
                direction = 0

        end_x = init_x + 2 * self.width
        if self.sections == 1:
            end_x = init_x + self.width
        end_y = init_y + self.sections * self.steps + 1
        if self.sections % 2 != 0:
            end_z = init_z + self.steps
        else:
            end_z = init_z

        self._end_position = Vec3(end_x, end_y, end_z)



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

import math

import mcpi.block
import mcpi.entity
from mcpi.vec3 import Vec3

from mcthings.decorators.decorator import Decorator
from mcthings_extra.entity import Entity


class VillagerDecorator(Decorator):
    """
    An Entity Decorator to populate the Thing.

    Add a villager to Thing
    """

    @classmethod
    def decorate(cls, thing):
        """
        Add a villager in the center of the Thing

        :return:
        """

        thing_end = thing.end_position
        thing_start = thing.position

        center_pos_x = thing_start.x + math.floor((thing_end.x - thing_start.x) / 2)
        center_pos_y = thing_start.y + math.floor((thing_end.y - thing_start.y) / 2)
        center_pos_z = thing_start.z + math.floor((thing_end.z - thing_start.z) / 2)

        entity = Entity(Vec3(center_pos_x, center_pos_y, center_pos_z))
        entity.entity = mcpi.entity.VILLAGER
        entity.spawn()




from mcpi import block
from mcpi.vec3 import Vec3
from mcthings.scene import Scene
from mcthings.thing import Thing

from math import *

# Based on Rainbow code written by zhuowei and retrieved from URL below:
# http://www.minecraftforum.net/topic/1638036-my-first-script-for-minecraft-pi-api-a-rainbow/
from mcthings.world import World

colors = [14, 1, 4, 5, 3, 11, 10]


class Rainbow(Thing):

    height = 20
    """ Height of the rainbow """
    block = block.WOOL.id

    def build(self):
        mc = World.server
        mc.setBlocks(-64, 0, 0, 64, self.height + len(colors), 0, 0)
        for x in range(0, 128):
            for color in range(0, len(colors)):
                y = sin((x / 128.0) * pi) * self.height + color
                mc.setBlock(self.position.x + x - 64, int(y), self.position.z,
                            self.block, colors[len(colors) - 1 - color])
        end_x = self.position.x - 64 + 128
        self._end_position = Vec3(end_x, self.position.y, self.position.z)